#include "main.h"


SDL_Surface *surface;
GLfloat cnt1 = 0.0f;
GLfloat cnt2 = 0.0f;

int inMenu = TRUE;
int posMenu=0;
int contador=0;

int gameBgSoundON = FALSE;
int menuBgMusicON = FALSE;

Uint32 ticks;
float delta;
Uint32 last_ticks;

int newGame = TRUE;

//Prototipos
void drawInGame();
void drawInMenu();


void Quit(int codigoRetornado)
{
    /*
     * Limpar a janela 
     */
    SDL_Quit();

    /*
     * e sair devidamente 
     */
    exit(codigoRetornado);
}



int resizeWindow(int width, int height)
{
    /*
     * Altura / largura 
     */
    GLfloat ratio;

    printf("resizeWindow\n");
    /*
     * Protege contra divises por 0 
     */
    if (height == 0) {
	height = 1;
    }
    ratio = (GLfloat) width / (GLfloat) height;
    glViewport(0, 0, (GLsizei) width, (GLsizei) height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(45.0f, ratio, 0.1f, 100.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    return (TRUE);

}



int iniciaGL()
{
    glShadeModel(GL_SMOOTH);	/* Shadding suave */
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);	/* Cor do fundo */
    glClearDepth(1.0f);		/* Buffer de profundidade */
    glEnable(GL_DEPTH_TEST);	/* Activa teste de profundidade */
    glDepthFunc(GL_LEQUAL);	/* Tipo de teste */

    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    /*
     * glHint( GL_POINT_SMOOTH_HINT, GL_NICEST ); glHint(
     * GL_POLYGON_SMOOTH_HINT, GL_NICEST ); 
     */

    return (TRUE);
}


int main(int argc, char **argv)
{
    /*
     * Modos a passar a SDL_SetVideoMode 
     */
    int videoFlags;
    /*
     * Flag do loop principal 
     */
    int done = FALSE;
    /*
     * Estrutura com os eventos SDL 
     */
    SDL_Event event;
    /*
     * Informao sobre o display 
     */
    const SDL_VideoInfo *videoInfo;
    /*
     * A janela est activa ou no? 
     */
    int isActive = TRUE;


    /*
     * GLfloat dific = 0.1; 
     */

    /*
     * Inicializar o SDL 
     */
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
	fprintf(stderr, "Falhou a inializao da imagem: %s\n",
		SDL_GetError());
	Quit(1);
    }
    /*
     * Ir buscar informaes sobre a imagem 
     */
    videoInfo = SDL_GetVideoInfo();

    if (!videoInfo) {
	fprintf(stderr, "Falhou o pedido de informaes de modo: %s\n",
		SDL_GetError());
	Quit(1);
    }
    videoFlags = SDL_OPENGL;	/* Activa OpenGL na SDL */
    videoFlags |= SDL_GL_DOUBLEBUFFER;	/* Activa double buffering */
    videoFlags |= SDL_HWPALETTE;	/* Guarda a paleta em hardware */
    videoFlags |= SDL_RESIZABLE;	/* Activa resize da janela */

    /*
     * Verifica se as texturas podem ser guardadas em Hardware 
     */
    if (videoInfo->hw_available)
	videoFlags |= SDL_HWSURFACE;
    else
	videoFlags |= SDL_SWSURFACE;

    /*
     * Verifica se podemos fazer Hardware blitting 
     */
    if (videoInfo->blit_hw)
	videoFlags |= SDL_HWACCEL;

    /*
     * Configura o doule buffering 
     */
    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

    /*
     * Cria uma superficie SDL 
     */
    surface =
	SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP,
			 videoFlags);
    resetChoques();
    /*
     * Verifica se a superficie foi criada 
     */
    if (!surface) {
	fprintf(stderr, "Falhou a criao da superficie: %s\n",
		SDL_GetError());
	Quit(1);
    }


    /*
     * Inicializa o OpenGL 
     */
    iniciaGL();

    /*
     * Faz o resize da janela inicial 
     */
    resizeWindow(SCREEN_WIDTH, SCREEN_HEIGHT);

    /*
     * Inicia as nossas coisas (a bola, o pad e o som) 
     */
    initBola();
    initPad();
    initParede();
    buildFont();

    /*
     * Verifica os parametros da linha de commandos e activa ou desactiva
     * o som e o debug
     */
    if (!(argc > 1 && (strcmp(argv[1], "--nosound")
		       || strcmp(argv[2], "--nosound")) == 0))
	initAudio();

    printf("%s\n", glGetString(GL_RENDERER));
    printf("%s\n", glGetString(GL_VENDOR));
    printf("%s\n", glGetString(GL_EXTENSIONS));

    /*
     * Loop principal 
     */
    last_ticks = SDL_GetTicks();
    while (!done) {

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glColor4f(1.0, 1.0, 1.0, 1.0);

	/*
	 * Tratar dos eventos em espera 
	 */
	while (SDL_PollEvent(&event)) {
	    switch (event.type) {
	    case SDL_ACTIVEEVENT:
		/*
		 * Aconteceu algo com o focus da nossa janela. Se perdemos 
		 * o focus ou a janela foi minimizada no devemos estar a
		 * desenhar o ecran 
		 */
		if (event.active.gain == 0)
		    isActive = FALSE;
		else
		    isActive = TRUE;
		break;
	    case SDL_VIDEORESIZE:
		/*
		 * Tratar do resize da janela 
		 */
		surface = SDL_SetVideoMode(event.resize.w,
					   event.resize.h, 16, videoFlags);
		if (!surface) {
		    fprintf(stderr,
			    "Could not get a surface after resize: %s\n",
			    SDL_GetError());
		    Quit(1);
		}
		resizeWindow(event.resize.w, event.resize.h);
		break;
	    case SDL_KEYDOWN:
		/*
		 * Tratar do teclado 
		 */
		if (carregaTeclado(&event.key.keysym) == 1) {
		    SDL_WM_ToggleFullScreen(surface);
		}
		break;
	    case SDL_KEYUP:
		largaTeclado(&event.key.keysym);
		break;
	    case SDL_QUIT:
		/*
		 * Tratar de pedidos de quit 
		 */
		done = TRUE;
		break;
	    default:
		break;
	    }
	}
	if (inMenu) {
          drawInMenu();
	} else {
	    
	  drawInGame();
	}

	glFlush();
	SDL_GL_SwapBuffers();
	/*
	 * Este delay causa problemas (apesar de aparentar ser portvel)
	 *
	 SDL_Delay(100); */
	SDL_Delay(1);
    }

    /*
     * Tratar das limpezas e sair 
     */
    Quit(0);

    /*
     * No devemos aqui chegar nunca, mas fica sempre bem :) 
     */

    return (0);
}


void drawInMenu()
{
    //Aqui fica a tralha que  desenhada no menu  
    //Verificar se saimos de um jogo e fazer reset ao mundo
    if (!newGame) {
	initBola();
	initPad();
	initParede();
	newGame = TRUE;
    }
    //Verificar se estamos a tocar o som de jogo e desligar
    if (gameBgSoundON) {
	Mix_HaltChannel(-1);

	gameBgSoundON = FALSE;
    }
    //Ligar a musica
    if (!menuBgMusicON) {
	ingameMusic();
	menuBgMusicON = TRUE;
    }
    

    ticks = SDL_GetTicks();
    delta = (ticks - last_ticks) / 1000.0;
    actualizaBola(delta);
    last_ticks = ticks;
    desenhaParede();
    desenhaBola();
    desenhaPad(1);
    intelX(1);
    intelY(1);
    actualizaPad(1);
    intelX(0);
    intelY(0);
    desenhaPad(0);
    actualizaPad(0);

   
    glLoadIdentity();
    glTranslatef(0.0f, 0.0f, -1.0f);
    // Variar a cor em funo da posio 
    glRasterPos2f(-0.3,0.3);

    glPrint("Bem vindo ao GlPong");   
    
    glRasterPos2f(-0.1,0.2f);


    glPrint("Menu", cnt1);
    
    glRasterPos2f(-0.15f,0.05f);
    glPrint("1 - Novo Jogo", cnt1);  
    
    glRasterPos2f(-0.225f,-0.15f);
    glPrint("Numero de choques: %d", getChoques() );  
    
    glRasterPos2f(-0.15f,-0.3f);
    glPrint("0 - Sair");
    
    glEnd();


     //Incrementa os contadores de posio e cor
    cnt1 += 0.051f;
    cnt2 += 0.005f;
}

void drawInGame()
{

    if (newGame) {
	initBola();
	initPad();
	initParede();
	newGame = FALSE;
    }
    //Aqui fica a tralha que  desenhada no jogo
    /*
     * Isto agora calcula bem o delta.
     *  
     */
    //Desliga a musica do jogo
    if (menuBgMusicON) {
	Mix_HaltMusic();
	menuBgMusicON = FALSE;
    }
    if (!gameBgSoundON) {
	ingameSound();
	gameBgSoundON = TRUE;
    }


    ticks = SDL_GetTicks();
    delta = (ticks - last_ticks) / 1000.0;
    /*      printf("ticks: %d | last_ticks: %d | delta: %f\n", ticks,
       last_ticks, delta); */

    actualizaBola(delta);

    last_ticks = ticks;

    desenhaParede();
    desenhaBola();

    /*
     * Desenha o pad do computador 
     */

    desenhaPad(1);
    intelX(1);
    intelY(1);

    actualizaPad(1);
    /*
     * actualizaPad( 1 ,intelX(getXpad(1),dific,delta),
     * intelY(getYpad(1),dific,delta)); 
     */

    desenhaPad(0);
    actualizaPad(0);



    /*
     * actualizaPad( 0 , getXpad(0),getYpad(0)); printf("pad 0 --> X = 
     * %f , Y= %f pad 1 --> x=%f , y=%f \n", getXpad(0), getYpad(0),
     * getXpad(1), getYpad(1)); 
     */

}
